@@ -16,7 +16,6 @@ limitations under the License.
 from __future__ import annotations
 
 import time
-import warnings
 from collections import namedtuple
 from dataclasses import dataclass
 from typing import Dict, List, Optional, Union
@@ -37,7 +36,6 @@ from cvxpy.interface.matrix_utilities import scalar_value
 from cvxpy.problems.objective import Maximize, Minimize
 from cvxpy.reductions import InverseData
 from cvxpy.reductions.chain import Chain
-from cvxpy.reductions.dgp2dcp.dgp2dcp import Dgp2Dcp
 from cvxpy.reductions.dqcp2dcp import dqcp2dcp
 from cvxpy.reductions.eval_params import EvalParams
 from cvxpy.reductions.flip_objective import FlipObjective
@@ -48,6 +46,7 @@ from cvxpy.reductions.solvers.conic_solvers.conic_solver import ConicSolver
 from cvxpy.reductions.solvers.defines import SOLVER_MAP_CONIC, SOLVER_MAP_QP
 from cvxpy.reductions.solvers.qp_solvers.qp_solver import QpSolver
 from cvxpy.reductions.solvers.solver import Solver
+from cvxpy.reductions.solvers.solver_inverse_data import SolverInverseData
 from cvxpy.reductions.solvers.solving_chain import (
     SolvingChain,
     construct_solving_chain,
@@ -56,6 +55,8 @@ from cvxpy.settings import SOLVERS
 from cvxpy.utilities import debug_tools
 from cvxpy.utilities.citations import CITATION_DICT
 from cvxpy.utilities.deterministic import unique_list
+from cvxpy.utilities.solver_context import SolverInfo
+from cvxpy.utilities.warn import warn
 
 SolveResult = namedtuple(
     'SolveResult',
@@ -164,14 +165,14 @@ class Problem(u.Canonical):
         self._objective = objective
         # Raise warning if objective has too many subexpressions.
         if debug_tools.node_count(self._objective) >= debug_tools.MAX_NODES:
-            warnings.warn("Objective contains too many subexpressions. "
-                          "Consider vectorizing your CVXPY code to speed up compilation.")
+            warn("Objective contains too many subexpressions. "
+                  "Consider vectorizing your CVXPY code to speed up compilation.")
         self._constraints = [_validate_constraint(c) for c in constraints]
         # Raise warning if constraint has too many subexpressions.
         for i, constraint in enumerate(self._constraints):
             if debug_tools.node_count(constraint) >= debug_tools.MAX_NODES:
-                warnings.warn(f"Constraint #{i} contains too many subexpressions. "
-                              "Consider vectorizing your CVXPY code to speed up compilation.")
+                warn(f"Constraint #{i} contains too many subexpressions. "
+                      "Consider vectorizing your CVXPY code to speed up compilation.")
 
         self._value = None
         self._status: Optional[str] = None
@@ -188,6 +189,9 @@ class Problem(u.Canonical):
         # Needed for _aggregate_metrics.
         self.ndim = 0
 
+        # solver_context : The solver context: supported constrains and bounds.
+        self.solver_context : Optional[SolverInfo] = None
+
     @perf.compute_once
     def _aggregate_metrics(self) -> dict:
         """
@@ -367,15 +371,51 @@ class Problem(u.Canonical):
     @perf.compute_once
     def is_qp(self) -> bool:
         """Is problem a quadratic program?
+
+        A problem is a QP if:
+        - It is DCP
+        - The objective is quadratic or piecewise-affine (QPWA)
+        - Inequality constraints (Inequality, NonPos, NonNeg) have PWL expressions
+        - Equality constraints (Equality, Zero) are allowed (DCP ensures affine args)
+        - No other constraint types (e.g., SOC, PSD, ExpCone) are present
+        - No PSD/NSD/Hermitian variables
         """
         for c in self.constraints:
-            if not (isinstance(c, (Equality, Zero)) or c.args[0].is_pwl()):
+            if type(c) in (Inequality, NonPos, NonNeg):
+                if not c.expr.is_pwl():
+                    return False
+            elif type(c) not in (Equality, Zero):
+                # Reject conic constraints (SOC, PSD, ExpCone, etc.)
                 return False
         for var in self.variables():
-            if var.is_psd() or var.is_nsd():
+            if var.attributes['PSD'] or var.attributes['NSD'] or var.attributes['hermitian']:
                 return False
         return (self.is_dcp() and self.objective.args[0].is_qpwa())
 
+    @perf.compute_once
+    def is_lp(self) -> bool:
+        """Is problem a linear program?
+
+        A problem is an LP if:
+        - It is DCP
+        - The objective is piecewise linear (PWL expressions linearize)
+        - Inequality constraints (Inequality, NonPos, NonNeg) have PWL expressions
+        - Equality constraints (Equality, Zero) are allowed (DCP ensures affine args)
+        - No other constraint types (e.g., SOC, PSD, ExpCone) are present
+        - No PSD/NSD/Hermitian variables
+        """
+        for c in self.constraints:
+            if type(c) in (Inequality, NonPos, NonNeg):
+                if not c.expr.is_pwl():
+                    return False
+            elif type(c) not in (Equality, Zero):
+                # Reject conic constraints (SOC, PSD, ExpCone, etc.)
+                return False
+        for var in self.variables():
+            if var.attributes['PSD'] or var.attributes['NSD'] or var.attributes['hermitian']:
+                return False
+        return (self.is_dcp() and self.objective.args[0].is_pwl())
+
     @perf.compute_once
     def is_mixed_integer(self) -> bool:
         return any(v.attributes['boolean'] or v.attributes['integer']
@@ -761,6 +801,7 @@ class Problem(u.Canonical):
             self._cache.key = key
             self._cache.solving_chain = solving_chain
             self._solver_cache = {}
+            self.solver_context = solving_chain.solver_context
         else:
             solving_chain = self._cache.solving_chain
 
@@ -773,18 +814,11 @@ class Problem(u.Canonical):
                 s.LOGGER.info(
                         'Using cached ASA map, for faster compilation '
                         '(bypassing reduction chain).')
-            if gp:
-                dgp2dcp = self._cache.solving_chain.get(Dgp2Dcp)
-                # Parameters in the param cone prog are the logs
-                # of parameters in the original problem (with one exception:
-                # parameters appearing as exponents (in power and gmatmul
-                # atoms) are unchanged.
-                old_params_to_new_params = dgp2dcp.canon_methods._parameters
-                for param in self.parameters():
-
-                    if param in old_params_to_new_params:
-                        old_params_to_new_params[param].value = np.log(
-                            param.value)
+            # Update parameter values for reductions that transform them.
+            # Each reduction handles its own parameter transformations
+            # (e.g., Dgp2Dcp applies log(), Complex2Real splits into real/imag).
+            for reduction in solving_chain.reductions:
+                reduction.update_parameters(self)
 
             data, solver_inverse_data = solving_chain.solver.apply(
                 self._cache.param_prog)
@@ -823,6 +857,8 @@ class Problem(u.Canonical):
                 # the last datum in inverse_data corresponds to the solver,
                 # so we shouldn't cache it
                 self._cache.inverse_data = inverse_data[:-1]
+        # Convert last inverse data (which is from the solver) to a SolverInverseData object.
+        inverse_data[-1] = SolverInverseData(inverse_data[-1], solving_chain.solver, solver_opts)
         return data, solving_chain, inverse_data
 
     def _find_candidate_solvers(self,
@@ -893,25 +929,14 @@ class Problem(u.Canonical):
                 candidates['qp_solvers'] = []  # No QP solvers allowed
 
         if self.is_mixed_integer():
-            # ECOS_BB must be called explicitly.
-            if slv_def.INSTALLED_MI_SOLVERS == [s.ECOS_BB] and solver != s.ECOS_BB:
-                msg = """
-
-                    You need a mixed-integer solver for this model. Refer to the documentation
-                        https://www.cvxpy.org/tutorial/advanced/index.html#mixed-integer-programs
-                    for discussion on this topic.
-
-                    Quick fix 1: if you install the python package CVXOPT (pip install cvxopt),
-                    then CVXPY can use the open-source mixed-integer linear programming
-                    solver `GLPK`. If your problem is nonlinear then you can install SCIP
-                    (pip install pyscipopt).
-
-                    Quick fix 2: you can explicitly specify solver='ECOS_BB'. This may result
-                    in incorrect solutions and is not recommended.
-                """
-                raise error.SolverError(msg)
-            # TODO: provide a useful error message when the problem is nonlinear but
-            #  the only installed mixed-integer solvers are MILP solvers (e.g., GLPK_MI).
+            if solver is None and not self.is_lp():
+                # Problem is mixed integer but not LP (e.g., MIQP, MISOCP)
+                # HiGHS only supports MILP, so warn users about MINLP solvers
+                warn(
+                    "Your problem is mixed-integer but not an LP. "
+                    "If your problem is nonlinear, consider installing SCIP "
+                    "(pip install pyscipopt) to solve it."
+                )
             candidates['qp_solvers'] = [
                 s for s in candidates['qp_solvers']
                 if slv_def.SOLVER_MAP_QP[s].MIP_CAPABLE]
@@ -925,7 +950,6 @@ class Problem(u.Canonical):
                     "QP/Conic solvers (%s) are not MIP-capable." %
                     (candidates['qp_solvers'] +
                      candidates['conic_solvers']))
-
         return candidates
 
     def _add_custom_solver_candidates(self, custom_solver: Solver):
@@ -1321,19 +1345,20 @@ class Problem(u.Canonical):
         backward_cache = self._solver_cache[s.DIFFCP]
         DT = backward_cache["DT"]
         zeros = np.zeros(backward_cache["s"].shape)
+        # del_vars: dictionary of variable gradients (delta/∂ with respect to variables)
+        # Maps variable IDs to their gradient arrays for the backward pass
         del_vars = {}
 
-        gp = self._cache.gp()
         for variable in self.variables():
             if variable.gradient is None:
                 del_vars[variable.id] = np.ones(variable.shape)
             else:
                 del_vars[variable.id] = np.asarray(variable.gradient,
                                                    dtype=np.float64)
-            if gp:
-                # x_gp = exp(x_cone_program),
-                # dx_gp/d x_cone_program = exp(x_cone_program) = x_gp
-                del_vars[variable.id] *= variable.value
+            # Apply chain rule through reductions that transform variables
+            for reduction in self._cache.solving_chain.reductions:
+                del_vars[variable.id] = reduction.var_backward(
+                    variable, del_vars[variable.id])
 
         dx = self._cache.param_prog.split_adjoint(del_vars)
         start = time.time()
@@ -1342,29 +1367,18 @@ class Problem(u.Canonical):
         backward_cache['DT_TIME'] = end - start
         dparams = self._cache.param_prog.apply_param_jac(dc, -dA, db)
 
-        if not gp:
-            for param in self.parameters():
-                param.gradient = dparams[param.id]
-        else:
-            dgp2dcp = self._cache.solving_chain.get(Dgp2Dcp)
-            old_params_to_new_params = dgp2dcp.canon_methods._parameters
-            for param in self.parameters():
-                # Note: if param is an exponent in a power or gmatmul atom,
-                # then the parameter passes through unchanged to the DCP
-                # program; if the param is also used elsewhere (not as an
-                # exponent), then param will also be in
-                # old_params_to_new_params. Therefore, param.gradient =
-                # dparams[param.id] (or 0) + 1/param*dparams[new_param.id]
-                #
-                # Note that param.id is in dparams if and only if
-                # param was used as an exponent (because this means that
-                # the parameter entered the DCP problem unchanged.)
-                grad = 0.0 if param.id not in dparams else dparams[param.id]
-                if param in old_params_to_new_params:
-                    new_param = old_params_to_new_params[param]
-                    # new_param.value == log(param), apply chain rule
-                    grad += (1.0 / param.value) * dparams[new_param.id]
-                param.gradient = grad
+        # Compute gradients for each parameter, applying chain rule through
+        # reductions that transform parameters (e.g., DGP log transform,
+        # Complex2Real split into real/imag).
+        for param in self.parameters():
+            # Start with the direct gradient if the param passed through unchanged
+            grad = 0.0 if param.id not in dparams else dparams[param.id]
+            # Apply chain rule through any reductions that transformed this param
+            for reduction in self._cache.solving_chain.reductions:
+                reduction_grad = reduction.param_backward(param, dparams)
+                if reduction_grad is not None:
+                    grad = grad + reduction_grad
+            param.gradient = grad
 
     def derivative(self) -> None:
         """Apply the derivative of the solution map to perturbations in the Parameters
@@ -1423,32 +1437,24 @@ class Problem(u.Canonical):
         D = backward_cache["D"]
         param_deltas = {}
 
-        gp = self._cache.gp()
-        if gp:
-            dgp2dcp = self._cache.solving_chain.get(Dgp2Dcp)
-
         if not self.parameters():
             for variable in self.variables():
                 variable.delta = np.zeros(variable.shape)
             return
 
+        # Compute deltas for transformed parameters, applying chain rule through
+        # reductions that transform parameters (e.g., DGP log transform,
+        # Complex2Real split into real/imag).
         for param in self.parameters():
             delta = param.delta if param.delta is not None else np.zeros(param.shape)
-            if gp:
-                if param in dgp2dcp.canon_methods._parameters:
-                    new_param_id = dgp2dcp.canon_methods._parameters[param].id
-                else:
-                    new_param_id = param.id
-                param_deltas[new_param_id] = (
-                    1.0/param.value * np.asarray(delta, dtype=np.float64))
-                if param.id in param_prog.param_id_to_col:
-                    # here, param generated a new parameter and also
-                    # passed through to the param cone prog unchanged
-                    # (because it was an exponent of a power)
-                    param_deltas[param.id] = np.asarray(delta,
-                                                        dtype=np.float64)
-            else:
+            # If param passed through unchanged, add its delta directly
+            if param.id in param_prog.param_id_to_col:
                 param_deltas[param.id] = np.asarray(delta, dtype=np.float64)
+            # Apply chain rule through any reductions that transformed this param
+            for reduction in self._cache.solving_chain.reductions:
+                transformed_deltas = reduction.param_forward(param, delta)
+                if transformed_deltas is not None:
+                    param_deltas.update(transformed_deltas)
         dc, _, dA, db = param_prog.apply_parameters(param_deltas,
                                                     zero_offset=True)
         start = time.time()
@@ -1459,10 +1465,9 @@ class Problem(u.Canonical):
             dx, [v.id for v in self.variables()])
         for variable in self.variables():
             variable.delta = dvars[variable.id]
-            if gp:
-                # x_gp = exp(x_cone_program),
-                # dx_gp/d x_cone_program = exp(x_cone_program) = x_gp
-                variable.delta *= variable.value
+            # Apply chain rule through reductions that transform variables
+            for reduction in self._cache.solving_chain.reductions:
+                variable.delta = reduction.var_forward(variable, variable.delta)
 
     def _clear_solution(self) -> None:
         for v in self.variables():
@@ -1527,7 +1532,6 @@ class Problem(u.Canonical):
             A solving chain that was used to solve the problem.
         inverse_data : list
             The inverse data returned by applying the chain to the problem.
-
         Raises
         ------
         cvxpy.error.SolverError
@@ -1536,13 +1540,13 @@ class Problem(u.Canonical):
 
         solution = chain.invert(solution, inverse_data)
         if solution.status in s.INACCURATE:
-            warnings.warn(
+            warn(
                 "Solution may be inaccurate. Try another solver, "
                 "adjusting the solver settings, or solve with "
                 "verbose=True for more information."
             )
         if solution.status == s.INFEASIBLE_OR_UNBOUNDED:
-            warnings.warn(INF_OR_UNB_MESSAGE)
+            warn(INF_OR_UNB_MESSAGE)
         if solution.status in s.ERROR:
             raise error.SolverError(
                     "Solver '%s' failed. " % chain.solver.name() +
@@ -1563,6 +1567,21 @@ class Problem(u.Canonical):
             for constr in self.constraints[1:]:
                 lines += [len(subject_to) * " " + str(constr)]
             return '\n'.join(lines)
+    
+    def format_labeled(self):
+        """Format problem with labels where available.
+        
+        Shows labels for both the objective expression and constraints.
+        """
+        if len(self.constraints) == 0:
+            return self.objective.format_labeled()
+        else:
+            subject_to = "subject to "
+            lines = [self.objective.format_labeled(),
+                     subject_to + self.constraints[0].format_labeled()]
+            for constr in self.constraints[1:]:
+                lines += [len(subject_to) * " " + constr.format_labeled()]
+            return '\n'.join(lines)
 
     def __repr__(self) -> str:
         return "Problem(%s, %s)" % (repr(self.objective),
